-- #hide
module HsUnixMisc where

import IO
import qualified Directory
import Posix
import GHC.IOBase

-- |
-- Format an @Int@ with leading zeros.
zeros :: Int            -- ^ how many characters to fill up
      -> Int            -- ^ value to represent as a string
      -> String         -- ^ string representation of the value, using the specified number of characters
zeros stellen z =
   let txt  = show z
       auff = stellen - length txt
       n    = take (if auff >= 0 then auff else 0) (repeat '0')
   in  n ++ txt


-- |
-- Remove trailing newlines. This is silimar to perl's @chomp@ procedure.
chomp :: String         -- ^ String to be chomped
      -> String         -- ^ same string, except for no newline characters at the end
chomp "" = ""
chomp "\n" = ""
chomp [x] = [x]
chomp (x:xs) = let xs' = chomp xs
               in  if xs' == "" && x == '\n' then "" else x:xs'


-- |
-- Get contents of a file or of @stdin@. This is @hGetContents@
-- applied either to a file or to @stdin@. A file name of
-- @\"-\"@ designates stdin. The contents are read lazily as
-- the string is evaluated.
--
-- The returned handle must be closed eventually, or the process will run
-- out of file handles.  
lazy_contents :: String                 -- ^ either the name of a file, or @\"-\"@
              -> IO (String, Handle)    -- ^ The contents of the file (or @stdin@) and the handle used to read it.
lazy_contents pfad = do
    h   <- if pfad == "-" then return stdin else openFile pfad ReadMode
    txt <- hGetContents h
    return (txt, h)

-- |
-- Get contents of a file or of @stdin@ eagerly. This is the
-- same as @lazy_contents@, except for the contents being
-- read immediately. 
contents :: String              -- ^ either the name of a file, or @\"-\"@ for @stdin@
         -> IO String           -- ^ the contents of the file or of standard input
contents pfad = do
    (txt, h) <- lazy_contents pfad
    seq (length txt) (return ())
    hClose h
    return txt


-- |
-- Test for the existence of a path. This is the disjunction of
-- @Directory.doesDirectoryExist@ and @Directory.doesFileExist@. For an dangling symlink, this will return @False@.
path_exists :: String    -- ^ Path
            -> IO Bool   -- ^ Whether the path exists in the file system
path_exists pfad = do
    de <- Directory.doesDirectoryExist pfad
    fe <- Directory.doesFileExist pfad
    return (de || fe)


-- |
-- Test if path points to a directory. This is a shortcut for
-- @Directory.doesDirectoryExist@.  
is_dir :: String        -- ^ Path
       -> IO Bool       -- ^ Whether the path exists and points to a directory.
is_dir = Directory.doesDirectoryExist


-- |
-- Test if path points to a file. This is a shortcut for
-- @Directory.doesFileExist@.  
is_file :: String       -- ^ Path
        -> IO Bool      -- ^ Whether the path exists and points to a file.
is_file = Directory.doesFileExist


-- |
-- This is the @System.Posix.Files.getFileStatus@ function from the GHC libraries, with improved error reporting. The GHC function doesn't include the
-- file name in the @IOError@ when the call fails, making error messages much less useful. @getFileStatus\'@ rectifies this.
--
-- See 'System.Posix.Files.getFileStatus'.
getFileStatus' :: FilePath              -- ^ Path of the file, whose status is to be queried
               -> IO FileStatus         -- ^ Status of the file
getFileStatus' path =
   getFileStatus path 
      `IO.catch` (\ioe -> ioError (ioe { ioe_filename = Just path }))
